/****************************************************************************
** Demonstration of moving plasma                                          **
**  by Steven H Don                                                        **
**                                                                         **
** For questions, feel free to e-mail me.                                  **
**                                                                         **
**    shd@earthling.net                                                    **
**    http://shd.cjb.net                                                   **
**                                                                         **
****************************************************************************/
#include "dos.h"
#include "math.h"

unsigned char *Screen = (unsigned char *)0xA0000000L;
unsigned char *Buffer;

//This sets the display to VGA 320x200 in 256 colours
void VGAScreen ()
{
  union REGS r;
  r.h.ah = 0;
  r.h.al = 0x13;
  int86(0x10, &r, &r);
}

//This resets the display to text mode
void TextScreen ()
{
  union REGS r;
  r.h.ah = 0;
  r.h.al = 0x3;
  int86(0x10, &r, &r);
}

//This sets a DAC register to a specific Red Green Blue-value
void SetDAC (unsigned char DAC, unsigned char R, unsigned char G, unsigned char B)
{
  outportb (0x3C8, DAC);
  outportb (0x3C9, R);
  outportb (0x3C9, G);
  outportb (0x3C9, B);
}

//Sets up the (real ugly) palette
void SetPalette ()
{
  unsigned char Counter;

  for (Counter = 0; Counter < 64; Counter++)
  {
    //Purple
    SetDAC (Counter      , Counter, 0, Counter);
    SetDAC (Counter +  64, 63 - Counter, 0, 63 - Counter);
    //Yellow
    SetDAC (Counter + 128, Counter, Counter, 0);
    SetDAC (Counter + 192, 63 - Counter, 63 - Counter, 0);
  }
}

//Draws the actual plasma
void DrawPlasma (int Phase)
{
  //Sine waves
  int Hor [320], Ver [200], Pos;
  //For angles
  double p, t, Radians;
  //Locations on screen
  int x, y;
  //Offset into video memory
  unsigned int o;

  Radians = (double)Phase / 5.0;

  /*
    Experiment with the waves, varying the formula the calculates "t"
    to get different kinds of waves.
  */

  //Calculate sine waves
  for (Pos = 0; Pos < 320; Pos++) {
    p = (double)Pos;
    //Put any sine wave you like here
    t = 35.0 * sin (cos (Radians / 2.0) * p / 70.0);
    t -= 50.0 * cos (Radians + (200.0 - p) / 40.0);
    Hor [Pos] = (int)t + 100;
  }

  Radians = (double)Phase / 10.0;
  for (Pos = 0; Pos < 200; Pos++) {
    p = (double)Pos;
    //And another one here
    t = 40.0 * cos (sin (Radians) * p / 25.0);
    t += 70.0 * sin (Radians + (200.0 - p) / 30.0);
    Ver [Pos] = (int)t + 160;
  }

  //Draw the entire plasma
  o = 0;
  for (y = 0; y < 200; y++) for (x = 0; x < 320; x++)
    Buffer [o++] = (x - Hor [x]) + (y - Ver [y]) + Phase;
}

void main ()
{
  int Frame = 0;

  //Switch to VGA
  VGAScreen ();

  //I'm drawing to the screen instead of a buffer
  Buffer = Screen;

  //Sets an ugly palette
  SetPalette ();

  //Draw the actual plasma
  do
    DrawPlasma (Frame++);
  while (!kbhit ());

  //And finally, return to text
  TextScreen ();
}